var Etcd = require('./promise.etcd');
var Q = require('q');
var _ = require('underscore');
var HashRing = require('hashring');

function Frodo(config) {
  this.config = config || {};
  this.context = this.config.context || 'frodo';
  this.ready = Q();
  var rawServers = this.config.etcdServers || [
    { host: '127.0.0.1', port: 4001 }
  ];
  if (!_.isArray(rawServers)) {
    rawServers = [ rawServers ];
  }
  var servers = [];
  rawServers.forEach(function (server) {
    if (_.isString(server)) {
      servers.push(server + ':4001');
    } else if (_.isObject(server)) {
      servers.push('' + (server.host || '127.0.0.1') + ':' + (server.port || '4001'));
    } else {
      throw new Error(server + ' as a server should be an address <string> or { host: <string>, port: <integer> }')
    }
  });

  this.etcd = new Etcd(servers, config.sslopts);

  if (!this.config.lazy) {
    this.ready = this.init();
  }
}

function addToMaps(node) {
  this.hosts.push(node.value);
  if (!this.hostsToKeys[node.value]) {
    this.hostsToKeys[node.value] = [];
  }
  this.hostsToKeys[node.value].push(node.key);
}

Frodo.prototype.init = function () {
  var self = this;
  self.pid = process.pid;
  self.hosts = [];
  self.hostsToKeys = {};
  self.hashRing = new HashRing(self.hosts, 'md5');
  self.hostsWatcher = self.etcd.watcher(self.context + '/hosts', null, { recursive: true });
  self.hostsWatcher.on('change', function (result) {
    if (result.action == 'set' && result.node.value) {
      addToMaps.call(self, result.node);

      self.hashRing.add(result.node.value);
    } else if (result.action == 'delete' && result.prevNode.value) {
      var hostsIndexOf = self.hosts.indexOf(result.prevNode.value);
      var hostsToKeysIndexOf = self.hostsToKeys[result.prevNode.value].indexOf(result.prevNode.key);

      if (hostsIndexOf >= 0) {
        self.hosts.splice(hostsIndexOf, 1);
        if (!self.hostsToKeys[result.prevNode.value] || self.hostsToKeys[result.prevNode.value].length == 1) {
          self.hashRing.remove(result.prevNode.value);
        }
      }

      if (hostsToKeysIndexOf >= 0) {
        self.hostsToKeys[result.prevNode.value].splice(hostsToKeysIndexOf, 1);
      }
    }
  });

  return self.etcd.get(self.context + '/hosts', { recursive: true }).then(function (result) {
    if (result && result[0] && result[0].node.nodes) {
      result[0].node.nodes.forEach(function (node) {
        addToMaps.call(self, node);
      });
      self.hosts = _.pluck(result[0].node.nodes, 'value');
      self.hashRing.add(self.hosts);
    }
  }, function (err) {
    if (err.errorCode == 100) {
      return self.etcd.mkdir(self.context + '/hosts/');
    } else {
      return Q.reject(err);
    }
  });
};

Frodo.prototype.addServer = function (server, options) {
  var self = this;
  var options = options || {};
  return this.ready.then(function () {
    return self.etcd.set(self.context + '/hosts/' + server + ':' + (options.id ? options.id : self.pid), server);
  });
};

Frodo.prototype.removeServer = function (server, options) {
  var self = this;
  var options = options || {};
  return this.ready.then(function () {
    if (options.all && self.hostsToKeys[server]) {
      return Q.all(self.hostsToKeys[server].map(function (key) {
        return self.etcd.del(key)
      }));
    } else {
      return self.etcd.del(self.context + '/hosts/' + server + ':' + (options.id ? options.id : self.pid));
    }
  });
};

Frodo.prototype.servers = function () {
  var self = this;
  return this.ready.then(function () {
    return self.hosts;
  });
};

Frodo.prototype.serverById = function (id) {
  var self = this;
  return this.ready.then(function () {
    return self.hashRing.get(id);
  });
};

Frodo.prototype.close = function () {
  this.hostsWatcher.stop();
  return Q();
};

module.exports = Frodo;
